/* 
 * Exemple de placage de textures.
 * Auteur : Fares BELHADJ (Lire COPYING)
 * Email  : amsi@ai.univ-paris8.fr
 * Date   : 17/10/2013
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#if defined(__APPLE__) || defined(MACOSX)
# include <GLUT/glut.h> /* Ils auraient pu m'eviter ca ! */
#else
# include <GL/glut.h>
#endif
#include "image.h"

static        GLuint texId = 0, pause = 0, linearInt = 0;
static        void   glutInitGL   (void);
static        void   glutReshape  (int l, int h);
static        void   glutDisplay  (void);
static        void   plane        (void);
static        void   glutIdle     (void);
static        void   glutKeyboard (unsigned char k, int x, int y);

/* un tout petit main */
int main(int argc, char ** argv) {
  int w = 640, h = 480;
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(w, h);
  glutCreateWindow("Hello le world !");
  glutInitGL();
  glutDisplayFunc(glutDisplay);
  glutIdleFunc(glutIdle);
  glutReshapeFunc(glutReshape);
  glutKeyboardFunc(glutKeyboard);
  glutMainLoop();
  return 0;
}

/* 
 * glutInitGL : configurer les parametres GL.
 */
static void glutInitGL(void) {
  image_t image;
  GLfloat light_ambient0[]  = { 0.4, 0.4, 0.4, 1.0 }; 
  glShadeModel(GL_SMOOTH);
  glDepthFunc(GL_LEQUAL);
  glClearColor(0, 0, 0, 0);
  glClearDepth(1.0);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glDisable(GL_CULL_FACE);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_NORMALIZE);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glLightfv (GL_LIGHT0, GL_AMBIENT,  light_ambient0);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  /* Cration de la texture */
  glGenTextures(1, &texId);
  glBindTexture(GL_TEXTURE_2D, texId); 
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  if(loadBMP("rgb.bmp", &image)) {
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.w, image.h, 0, GL_BGR, GL_UNSIGNED_BYTE, image.data);
    free(image.data);
  } else
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 2, 2, 0, GL_BGR, GL_UNSIGNED_BYTE, NULL);
}

/* 
 * glutReshape : modifier la matrice de projection selon le
 * redimensionnement de la fenetre glut.
 */
static void glutReshape(int l, int h) {
  glViewport(0, 0, l, h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-1.5, 1.5, -1.5, 1.5, 2.5, 100.0);
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity( );
}

/*
 * glutIdle : fonction appelee a chaque tour de boucle dans
 * glutLoop().
 */
static void glutIdle(void) {
  glutPostRedisplay();
}

static void glutKeyboard(unsigned char k, int x, int y) {
  switch(k) {
  case 'l':
    if(glIsEnabled(GL_LIGHTING)) {
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
    } else {
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);
    }
    break;
  case 'i':
    linearInt = !linearInt;
    glBindTexture(GL_TEXTURE_2D, texId); 
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, linearInt ? GL_LINEAR : GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, linearInt ? GL_LINEAR : GL_NEAREST);
    break;
  case ' ':
  case 'p':
    pause = !pause;
    break;
  case 27:
  case 'q':
    exit(0);
  default:
    break;
  }
}

/*
 * glutDisplay : calculer une rotation puis une translation dans le repere
 * MODELVIEW puis dessiner le plan textur.
 */
static void glutDisplay(void) {
  static float a = 0.0;
  static int t0 = 0, t1;
  GLdouble dt;
  /* Calcul du dt */
  t1 = glutGet(GLUT_ELAPSED_TIME);
  dt = (t1 - t0) / 1000.0;
  t0 = t1;
  /* Fin du calcul du dt */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();
  glTranslatef(0.0f, 0.0f, -3.5f);
  glRotatef(a, 0, 1, 0);
  plane();
  glutSwapBuffers();
  if(!pause)
    a += 90 * dt;
}

/*\!brief dessine un plan unitaire avec normales et coordonnes de
 * texture ; cette fonction utilise des QUADS.
 */
static void plane(void) {
  GLfloat vertices[] = {
    -1.0f, 1.5f, 0.0f, 
    -1.0f,-1.5f, 0.0f, 
     1.0f,-1.0f, 0.0f, 
     1.0f, 1.0f, 0.0f }; 
  GLfloat normals[] = { 
    0.0f, 0.0f, 1.0f,
    0.0f, 0.0f, 1.0f,
    0.0f, 0.0f, 1.0f,
    0.0f, 0.0f, 1.0f
  };
  GLfloat coords[] = { 
    0.0f, 1.0f,
    0.0f, 0.0f,
    1.0f, 0.0f,
    1.0f, 1.0f
  };
  glBindTexture(GL_TEXTURE_2D, texId);
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  glVertexPointer(3, GL_FLOAT, 0, vertices);
  glNormalPointer(GL_FLOAT, 0, normals);
  glTexCoordPointer(2, GL_FLOAT, 0, coords);
  glDrawArrays(GL_QUADS, 0, 4);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
}
